# PiconCleaner
#
# Coded by dre (c) 2014 - 2018
# Support: board.dreamboxtools.de
# E-Mail: dre@dreamboxtools.de
#
# This plugin is open source but it is NOT free software.
#
# This plugin may only be distributed to and executed on hardware which
# is licensed by Dream Property.
# In other words:
# It's NOT allowed to distribute any parts of this plugin or its source code in ANY way
# to hardware which is NOT licensed by Dream Property.
# It's NOT allowed to execute this plugin and its source code or even parts of it in ANY way
# on hardware which is NOT licensed by Dream Property.
#
# If you want to use or modify the code or parts of it,
# you have to keep MY license and inform me about the modifications by mail.

from Components.ActionMap import ActionMap, HelpableActionMap
from Components.config import config
from Components.Label import Label
from Components.MultiContent import MultiContentEntryText, MultiContentEntryPixmapAlphaBlend
from Components.Pixmap import Pixmap
from Components.Sources.List import List
from Components.Sources.StaticText import StaticText
from Screens.ChoiceBox import ChoiceBox
from Screens.MessageBox import MessageBox
from Plugins.Plugin import PluginDescriptor
from Screens.ChannelSelection import ChannelSelectionBase
from Screens.Screen import Screen
from Screens.HelpMenu import HelpableScreen
from Tools.Directories import fileExists, SCOPE_SKIN_IMAGE, resolveFilename, pathExists
from Tools.LoadPixmap import LoadPixmap

from os import listdir as os_listdir, path as os_path
from enigma import eServiceCenter, eServiceReference, gFont, loadPNG, eEnv, RT_HALIGN_RIGHT, eConsoleAppContainer
from . import _

png_cache = {}

def PiconPixmap(picon, png_cache):
	png = png_cache.get(picon, None)
	if png is None: # no cached entry
		png = LoadPixmap(cached=False, path=picon)
		png_cache[picon] = png
	if png is None:
		png = png_cache.get("default", path=None)
		if png is None:
			png = LoadPixmap(cached=False, path=resolveFilename(SCOPE_SKIN_IMAGE, "skin_default/picon_default.png"))
			png_cache["default"] = png
	return png

def main(session,**kwargs):
	session.open(PiconCleaner)

def Plugins(**kwargs):

	list = []
	list.append(PluginDescriptor(name="PiconCleaner", description=_("Clean up unused picons"), where = [PluginDescriptor.WHERE_PLUGINMENU], icon="PiconCleaner.png", fnc=main))
	return list

class PiconCleaner(Screen, HelpableScreen):
	skin = """
		<screen position="center,center" size="1000,600" title="PiconCleaner">
			<widget source="menu" render="Listbox" position="0,0" size="100,560" scrollbarMode="showOnDemand"  alternate_row_color="#00333333">
				<convert type="TemplatedMultiContent">
					{"templates":
						{"default":( 25,
							[
								MultiContentEntryText(pos=(0,0), size=(200,25), font=0, text=0),
							]),
						},
						"fonts": [gFont("Regular",20)],
					}
				</convert>
			</widget>
			<widget source="list" render="Listbox" position="100,0" size="900,560" scrollbarMode="showOnDemand"  alternate_row_color="#00333333">
				<convert type="TemplatedMultiContent">
					{"templates":
						{"default":( 140,
							[
								MultiContentEntryPixmapAlphaBlend(pos=(0,4), size=(220,132), png=2), #index 2 is the picon
								MultiContentEntryText(pos=(230,5), size=(650,60), flags=RT_HALIGN_RIGHT|RT_WRAP, text=0), # index 0 is the path
								MultiContentEntryText(pos=(230,65), size=(650,30), flags=RT_HALIGN_RIGHT, text=1), # index 1 is the sref
							]),
						 },
						"fonts": [gFont("Regular",20)],
					}
				</convert>
			</widget>
			<widget name="ButtonRedtext" position="30,570" size="300,25" valign="center" halign="left" zPosition="10" font="Regular;18" transparent="1" />
			<widget name="ButtonRed" pixmap="skin_default/buttons/button_red.png" position="10,572" zPosition="10" size="16,16" transparent="1" alphatest="on" />
			<widget name="ButtonBluetext" position="690,570" size="300,25" valign="center" halign="left" zPosition="10" font="Regular;18" transparent="1" />
			<widget name="ButtonBlue" pixmap="skin_default/buttons/button_blue.png" position="670,572" zPosition="10" size="16,16" transparent="1" alphatest="on" />
			<widget name="ButtonGreentext" position="360,570" size="300,25" valign="center" halign="left" zPosition="10" font="Regular;18" transparent="1" />
			<widget name="ButtonGreen" pixmap="skin_default/buttons/button_green.png" position="340,572" zPosition="10" size="16,16" transparent="1" alphatest="on" />
		</screen>
		"""

	def __init__(self, session, args=0):
		self.skin = PiconCleaner.skin
		self.session = session
		Screen.__init__(self, session)
		HelpableScreen.__init__(self)
		
		self["WizardActions"] = HelpableActionMap(self, "WizardActions",
		{
			"back":	(self.close, _("Close")),
			"up":		(self.moveUp,_("Move up")),
			"down":		(self.moveDown,_("Move down")),
			"left":		(self.pageUp,_("Page up")),
			"right":	(self.pageDown,_("Page down")),
		}, -1)

		self["ColorActions"] = HelpableActionMap(self, "ColorActions",
		{
			"red":	(self.deleteSelectedPicon, _("Delete selected picon")),
			"blue": (self.deleteAllPicons, _("Delete all redundant picons")),
			"green": (self.assignNewServiceReferenceCS, _("Rename selected picon")),
		}, -1)		
		
		self["ChannelSelectBaseActions"] = HelpableActionMap(self, "ChannelSelectBaseActions",
		{
			"nextMarker":	(self.nextCategory,_("Change to next size")),
			"prevMarker": 	(self.previousCategory,_("Change to previous size")),
			"nextBouquet":	(self.previousCategory,_("Change to previous size")),
			"prevBouquet": 	(self.nextCategory,_("Change to next size")),	
		}, -1)
		
		self["ButtonBlue"] = Pixmap()
		self["ButtonBluetext"] = Label(_("Delete all redundant picons"))
		
		self["ButtonRed"] = Pixmap()
		self["ButtonRedtext"] = Label(_("Delete selected picon"))

		self["ButtonGreen"] = Pixmap()
		self["ButtonGreentext"] = Label(_("Reassign selected picon"))
		
		self.list = []
		self["list"] = List(self.list)
		self.menuList = []
		self["menu"] = List(self.menuList)
		
		self.PiconEntryList = []
		self.allServicesList = []
		self.markerBouquetList = []
		self.stringServicesList = []
		self.categoryList = [_("all")]
		self.cmdList = []
		self.run = 0
		self.errorcounter = 0
		self.menuIndex = 0
		self.currentCategory = _("all")
				
		self.container = eConsoleAppContainer()
		self.appClosed_conn = self.container.appClosed.connect(self.runFinished)
		self.dataAvail_conn = self.container.dataAvail.connect(self.dataAvail)
		
		self.containerRename = eConsoleAppContainer()
		self.appClosed_connRename = self.containerRename.appClosed.connect(self.runFinishedRename)
		self.dataAvail_connRename = self.containerRename.dataAvail.connect(self.dataAvailRename)
		
		self.piconpathlist = [eEnv.resolve("${datadir}/enigma2/")]
		
		dirList = os_listdir("/media")
		
		blockList =  ["hdd"]

		if fileExists("/proc/mounts"):
			mountsFile = open("/proc/mounts" ,"r")
			for line in mountsFile:
				entry = line.split()
				if entry[2] in ["nfs", "nfs4", "smbfs", "cifs"]:
					if entry[1].startswith("/media/"):
						print "[PiconCleaner] - found network drive. Exclude ", entry[1]
						blockList.append(entry[1][7:])
			mountsFile.close()
		
		for dir in dirList:
			if dir in blockList:
				continue
			if os_path.ismount("/media/%s" %(dir)) or (os_path.islink("/media/%s" %(dir)) and os_path.ismount(os_path.realpath("/media/%s" %(dir)))):
				self.piconpathlist.append("/media/%s/" %(dir))
			
		if os_path.ismount("/data"):
			self.piconpathlist.append("/data/")
		if os_path.ismount("/dreambox-data"):
			self.piconpathlist.append("/dreambox-data/")		
		
		self.onShown.append(self.buildScreen)

	def pageUp(self):
		self["list"].moveSelection("pageUp")
		
	def pageDown(self):
		self["list"].moveSelection("pageDown")
		
	def moveUp(self):
		self["list"].moveSelection("moveUp")
		
	def moveDown(self):
		self["list"].moveSelection("moveDown")

	def nextCategory(self):
		if self.menuIndex<len(self.menuList)-1:
			self.menuIndex += 1
		else:
			self.menuIndex = 0
		self["menu"].setIndex(self.menuIndex)
		self.currentCategory = self["menu"].getCurrent()[0]
		self.buildScreen()		
			
	def previousCategory(self):
		if self.menuIndex>0:
			self.menuIndex -= 1
		else:
			self.menuIndex = len(self.menuList)-1
		self["menu"].setIndex(self.menuIndex)
		self.currentCategory = self["menu"].getCurrent()[0]
		self.buildScreen()
			
	def buildEntry(self, path, filename, piconpath, category):
		print "buildEntry called"
										
		png = png_cache.get(piconpath, None)
		if png is None:
			png = LoadPixmap(cached=False, path=piconpath)
			png_cache[piconpath] = png
		
		
		return [path, filename, png, category]
	
	def buildScreen(self):
		serviceslist = self.getBouquets()


		self.list = []
		self.menuList = []
		self.piconDict = {}
		
		for size in ['picon_50x30/','picon/','picon_220x132/','picon_320x240/','picon_400x170/','picon_400x140/','picon_500x300/','picon_oled/','picon_128x64/','picon_132x46/', 'picon_800x480/']:
			for piconpathentry in self.piconpathlist:
				path = piconpathentry + size

				if pathExists(path) and not os_path.islink(path[:-1]):
					sizeSplit = size.split("_")
					category=None
					if len(sizeSplit) == 2:
						category = sizeSplit[1][:-1]
					elif len(sizeSplit) == 1 and sizeSplit[0] == "picon/":
						category = "100x60"
					if category is not None and category not in self.categoryList:
						self.categoryList.append(category)
					for file in os_listdir(path):
						pos = file.find(".png")
						if pos != -1:
							filename = file[:pos]
						
							if not filename in serviceslist and not filename in self.markerBouquetList:
								piconpath = path+file
								
								# if category is not all the list of leftover picons can be built directly
								if self.currentCategory != _("all") and self.currentCategory == category:
									self.list.append(("%s" %(path), "%s" %(filename), piconpath, category))									
								
								# if category is all all paths are stored in a dict for later use
								if self.currentCategory == _("all"):
									currentValueList = self.piconDict.get(filename, [])
									currentValueList.append("%s" %(path))
									self.piconDict[filename] = currentValueList
									
		if self.currentCategory == _("all"):
			for key in self.piconDict.keys():
				path = ""
				pathSet = False
				for item in self.piconDict[key]:
					if path == "":
						path = item
					else:
						path = path + ", " + item
					# this sets the piconpath to the biggest size
					piconpath = item + key + ".png"
				
				self.list.append(("%s" %(path), "%s" %(key), piconpath, ""))
						
		self["list"].updateList(self.list)	
		self["list"].setBuildFunc(self.buildEntry)
		for cat in self.categoryList:
			self.menuList.append((cat,))
		"""
		if self.currentCategory == "50x30":
			self["list"].style = "50x30"
		elif self.currentCategory == _("all"):
			self["list"].style = "default"
		else:
			self["list"].style = "100x60"
		"""
		self["menu"].setList(self.menuList)
		self["menu"].setIndex(self.menuIndex)
		
	def deleteSelectedPicon(self):
		if len(self.list) > 0:
			self.index = self["list"].index
			self.cmdList = []
			paths = self.list[self.index][0].split(",")
			for path in paths:
				file = "%s%s.png" %(path, self.list[self.index][1])
				print "file", file
		
				file = file.replace("(", "\(").replace(")", "\)").replace(" ", "\ ")
		
				self.cmdList.append("rm -f %s" %(file))
			
			print "self.cmdLIst", self.cmdList
			print "run", self.run
			self.container.execute(self.cmdList[self.run])
			
	def assignNewServiceReferenceCS(self):
		if len(self.list) > 0:
			self.index = self["list"].index
			
			self.fileToRename = "%s.png" %(self.list[self.index][1])
			self.pathsForRename = self.list[self.index][0].split(",")
			self.session.openWithCallback(self.renamePicon, ReassignPiconCS,None)

	def renamePicon(self, ref, selectedType):
		if ref is not None:
			if selectedType == 0: # sRef
				# remove path 
				if ref.getPath():
					ref.setPath("")
				refString = ref.toString()
				x = refString.split(':')
				if len(x) < 11: # skip invalid service references # Dr. Best --> i am doing this the dmm-way now...
					newFileName = ""
				else:
					del x[x[10] and 11 or 10:] # remove name and empty path
					x[1]='0' #replace flags field
					newFileName = '_'.join(x).strip('_')

			else: # name
				eServiceCenterInstance = eServiceCenter.getInstance()
				info = eServiceCenterInstance.info(ref)
				newFileName = info.getName(ref).replace('\xc2\x86', '').replace('\xc2\x87', '').replace(" ", "_")
			
			print "newFileName", newFileName
			if newFileName != "":
				piconNotRenamed = False
				piconRenamed = False
		
				self.cmdList = []

				for path in self.pathsForRename:
					fileNew = "%s%s.png" %(path, newFileName)
					if fileExists(fileNew):
						piconNotRenamed = True
						continue
					else:
						oldfile = "%s%s" %(path, self.fileToRename.replace("(", "\(").replace(")", "\)").replace(" ", "\ "))
						self.cmdList.append("mv %s %s" %(oldfile, fileNew))
				
						print self.cmdList
		
				if len(self.cmdList)>0:
					self.containerRename.execute(self.cmdList[self.run])	
				else:
					self.session.open(MessageBox, _("Picon(s) already exist(s)"), MessageBox.TYPE_INFO, timeout=5)

	def runFinishedRename(self, returnvalue):
		print "[Picon Cleaner] - Command executed: %d" %(returnvalue)
		if returnvalue != 0:
			self.errorcounter += 1
		print "self.run", self.run			
		self.run += 1

		
		if self.run < len(self.cmdList):
			print "run the next"
			self.containerRename.execute(self.cmdList[self.run])
		else:
			if self.errorcounter == 0:
				print "no error"
				self.session.open(MessageBox, _("Picons renamed"), MessageBox.TYPE_INFO, timeout=5)
			else:
				print "error"
				self.session.open(MessageBox, _("%d/%d picons were renamed\n%d picons were not renamed due to an error") %(len(self.cmdList)-self.errorcounter, len(self.cmdList), self.errorcounter), MessageBox.TYPE_ERROR, timeout=5)
			
			self.errorcounter = 0
			self.run = 0
			self.cmdList = []				

	def deleteAllPicons(self):
		if len(self.list) > 0:
			for entry in self.list:
				paths = entry[0].split(",")
				for path in paths:
					file = "%s%s.png" %(path, entry[1])
			
					file = file.replace("(", "\(").replace(")", "\)").replace(" ", "\ ")
			
					self.cmdList.append("rm -f %s" %(file))
				
			print self.cmdList
			
			self.container.execute(self.cmdList[self.run])
		
	def getBouquetServices(self, bouquetRoot):
		bouquets = []
		
		self.eServiceCenterInstance = eServiceCenter.getInstance()
		if config.usage.multibouquet.value:
			serviceList = self.eServiceCenterInstance.list(bouquetRoot)
			if serviceList:
				while True:
					service = serviceList.getNext()
					if not service.valid():
						break
					if service.flags & eServiceReference.isDirectory:
						info = self.eServiceCenterInstance.info(service)
						if info:
							bouquets.append(service)
							self.markerBouquetList.append(info.getName(service).replace(" ", "_"))
				return bouquets
		else:
			info = self.eServiceCenterInstance().info(bouquetRoot)
			if info:
				bouquets.append(bouquetRoot)
			return bouquets
	
	def getBouquets(self):		
		from Screens.ChannelSelection import service_types_tv, service_types_radio
		bouquetStringList = []
		if config.usage.multibouquet.value:
			bouquetStringList.append(service_types_tv + ' FROM BOUQUET "bouquets.tv"')
			bouquetStringList.append(service_types_radio + ' FROM BOUQUET "bouquets.radio"')
		else:
			bouquetStringList.append(service_types_tv + ' FROM BOUQUET "userbouquet.favourites.tv"')
			bouquetStringList.append(service_types_radio + ' FROM BOUQUET "userbouquet.favourites.radio"')
			
		for bouquetString in bouquetStringList:
			bouquetRoot = eServiceReference(bouquetString)
			bouquetServices = self.getBouquetServices(bouquetRoot)
			if bouquetServices is None:
				continue
				
			for service in bouquetServices:
				self.getDirectoryServices(service)
			
			for service in self.allServicesList:
				sRef = self.buildServiceRefString(service)

				if sRef is "":
					continue

				self.stringServicesList.append(sRef)	

		return self.stringServicesList				
				
	def getDirectoryServices(self, directory):
		serviceList = self.eServiceCenterInstance.list(directory)
		
		if serviceList:
			while True:
				service = serviceList.getNext()
				oldPath = ""
				if service.getPath() and not service.flags & eServiceReference.isGroup and not service.flags & eServiceReference.isMarker:
					oldPath = service.getPath()
					service.setPath("")
				if not service.valid():
					service.setPath(oldPath)
					break
				if service.flags & eServiceReference.isGroup:
					self.getDirectoryServices(service)
				elif service.flags & eServiceReference.isMarker:
					self.markerBouquetList.append(service.getName().replace(" ", "_"))
				else:
					self.allServicesList.append(service)
				service.setPath(oldPath)

	def buildServiceRefString(self,service):
		sRef = service.toString()
		if sRef is "":
			return sRef

		# strip all after last http (e.g. partnerbox-services)
		pos = sRef.rfind("http")
		if pos != -1:
			sRef = sRef[:pos]

		pos = sRef.rfind(':')
		if pos == -1:
			return ""

		sRef = sRef[:pos].rstrip(':').replace(':','_')
		return sRef

	def dataAvail(self, output):
		print "[Picon Cleaner] - Data available: %s" %(output)

	def dataAvailRename(self, output):
		print "[Picon Cleaner] - Data available: %s" %(output)

	def runFinished(self, returnvalue):
		print "[Picon Cleaner] - Command executed: %d" %(returnvalue)
		if returnvalue != 0 and self.run>0:
			self.errorcounter += 1
			
		self.run += 1
		
		if self.run < len(self.cmdList):
			self.container.execute(self.cmdList[self.run])
		else:
			print "run has finished"
			if self.errorcounter == 0:
				self.session.open(MessageBox, _("Picons deleted"), MessageBox.TYPE_INFO, timeout=5)
			else:
				self.session.open(MessageBox, _("%d/%d picons were deleted\n%d picons were not deleted due to an error") %(len(self.cmdList)-self.errorcounter, len(self.cmdList), self.errorcounter), MessageBox.TYPE_ERROR, timeout=5)
			
			self.errorcounter = 0
			self.run = 0
			self.cmdList = []
			
class ReassignPiconCS(ChannelSelectionBase):
	skin = """
		<screen name="ReassignPiconCS" position="center,120" size="820,520" title="Reassign picon to...">
			<ePixmap pixmap="skin_default/buttons/red.png" position="10,5" size="200,40" alphatest="on" />
			<ePixmap pixmap="skin_default/buttons/green.png" position="210,5" size="200,40" alphatest="on" />
			<ePixmap pixmap="skin_default/buttons/yellow.png" position="410,5" size="200,40" alphatest="on" />
			<ePixmap pixmap="skin_default/buttons/blue.png" position="610,5" size="200,40" alphatest="on" />
			<widget source="key_red" render="Label" position="10,5" size="200,40" zPosition="1" font="Regular;20" halign="center" valign="center" backgroundColor="#9f1313" transparent="1" shadowColor="black" shadowOffset="-2,-2" />
			<widget source="key_green" render="Label" position="210,5" size="200,40" zPosition="1" font="Regular;20" halign="center" valign="center" backgroundColor="#1f771f" transparent="1" shadowColor="black" shadowOffset="-2,-2" />
			<widget source="key_yellow" render="Label" position="410,5" size="200,40" zPosition="1" font="Regular;20" halign="center" valign="center" backgroundColor="#a08500" transparent="1" shadowColor="black" shadowOffset="-2,-2" />
			<widget source="key_blue" render="Label" position="610,5" size="200,40" zPosition="1" font="Regular;20" halign="center" valign="center" backgroundColor="#18188b" transparent="1" shadowColor="black" shadowOffset="-2,-2" />
			<eLabel position="10,50" size="800,1" backgroundColor="grey" />
			<widget name="list" position="10,55" size="800,420" enableWrapAround="1" scrollbarMode="showOnDemand"  alternate_row_color="#00333333"/>
			<eLabel position="10,480" size="800,1" backgroundColor="grey" />
			<widget source="introduction" render="Label" position="10,488" size="800,25" font="Regular;22" halign="center" />
		</screen>"""

	def __init__(self, session, title):
		ChannelSelectionBase.__init__(self, session)
		self.onShown.append(self.__onExecCallback)

		self["actions"] = ActionMap(["OkCancelActions", "TvRadioActions", "ChannelSelectBaseActions"],
			{
				"showProviders": self.doNothing,
				"showSatellites": self.showAllServices,
				"showAllServices": self.cancel,
				"cancel": self.cancel,
				"ok": self.channelSelected,
				"keyRadio": self.setModeRadio,
				"keyTV": self.setModeTv
			})

		self["key_red"] = StaticText(_("Close"))
		self["key_green"] = StaticText(_("All"))
		self["key_yellow"] = StaticText()
		self["key_blue"] = StaticText(_("Favourites"))
		self["introduction"] = StaticText(_("Press OK to reassign picon"))

	def __onExecCallback(self):
		self.setModeTv()
		self.setTitle(_("Reassign picon to..."))

	def doNothing(self):
		pass

	def channelSelected(self): # just return selected service
		ref = self.getCurrentSelection()
		print "sel type", type(ref)
		
		if ref.getPath():
			hasServicePath = True
		if (ref.flags & 7) == 7:
			self.enterPath(ref)
		elif not (ref.flags & eServiceReference.isMarker):
			self.ref = self.getCurrentSelection()
			self.session.openWithCallback(self.closeScreen, ChoiceBox, _("Select what to use for the new picon name"), list=[(_("Service Reference"), 0), (_("Service Name"), 1)], windowTitle=_("PiconCleaner"), allow_cancel = False)

	def renamePiconTypeSelection(self):
		self.session.openWithCallback(self.changePiconName,ChoiceBox, _("Select what to use for the new picon name"), list=[(_("Service Reference"), 0), (_("Service Name"), 1)], windowTitle=_("PiconCleaner"), allow_cancel = False)
		
	def closeScreen(self, answer):
		self.close(self.ref, answer[1])	

	def setModeTv(self):
		self.setTvMode()
		self.showFavourites()

	def setModeRadio(self):
		self.setRadioMode()
		self.showFavourites()

	def cancel(self):
		self.close(None, None)